/*
 * Class created by lensfrex.
 */

/*
 * Class created by lensfrex.
 */

/*
 * Class created by lensfrex.
 */

/*
 * Class created by lensfrex.
 */

package net.lensfrex.dscape.exception.handler;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.NotRoleException;
import com.fasterxml.jackson.databind.JsonMappingException;
import net.lensfrex.dscape.dto.response.general.Response;
import net.lensfrex.dscape.dto.response.general.ResponseCode;
import net.lensfrex.dscape.exception.GlobalException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.multipart.MultipartException;

/**
 * 全局异常处理，负责后端抛出Exception时响应相应的信息
 */
@RestControllerAdvice
public class GlobalExceptionHandler {
    private static final Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    /**
     * 负责处理各种自定义业务异常（权限不足，学号不存在等）
     *
     * @param e 抛出的异常
     * @return 统一响应
     */
    @ExceptionHandler(GlobalException.class)
    public Response<Object> handler(GlobalException e) {
        log.debug("请求错误: " + e.getResponseCode().getMessage());
        return Response.error(e.getResponseCode());
    }

    /**
     * 负责处理和响应其他部分都没有接收的异常（说白了就是剩下的都返回50000）
     * 这部分日志要特别关注，所以以error打出来
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(Exception.class)
    public Response<Object> handler(Exception e) {
        log.error("请求错误: " + e.getMessage());
        log.error("异常类: " + e.getClass());
        log.error("追踪：", e);
        return Response.error(ResponseCode.SERVER_INTERNAL_ERROR);
    }

    /**
     * 处理参数不完整的请求异常
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(MissingServletRequestParameterException.class)
    public Response<Object> handler(MissingServletRequestParameterException e) {
        log.debug("请求的参数不完整: " + e.getMessage());
        return Response.error(ResponseCode.PARAM_WRONG);
    }

    /**
     * 处理参数类型错误的请求异常（请求参数类型错误）
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(MethodArgumentTypeMismatchException.class)
    public Response<Object> handler(MethodArgumentTypeMismatchException e) {
        log.debug(String.format("请求错误（%s）: %s", e.getClass().getName(), e.getMessage()));
        return Response.error(ResponseCode.PARAM_WRONG);
    }

    /**
     * 处理参数类型错误的请求异常（请求参数类型错误）
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(IllegalArgumentException.class)
    public Response<Object> handler(IllegalArgumentException e) {
        log.debug(String.format("请求错误（%s）: %s", e.getClass().getName(), e.getMessage()));
        return Response.error(ResponseCode.PARAM_WRONG);
    }

    /**
     * 处理参数类型错误的请求异常2（Json解析错误）
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(JsonMappingException.class)
    public Response<Object> handler(JsonMappingException e) {
        log.debug(String.format("请求错误（%s）: %s", e.getClass().getName(), e.getMessage()));
        return Response.error(ResponseCode.PARAM_WRONG);
    }

    /**
     * 处理参数类型错误的请求异常3（字段映射错误）
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(HttpMessageNotReadableException.class)
    public Response<Object> handler(HttpMessageNotReadableException e) {
        log.debug(String.format("请求错误（%s）: %s", e.getClass().getName(), e.getMessage()));
        return Response.error(ResponseCode.PARAM_WRONG);
    }

    /**
     * 处理请求头中“Content-Type”字段不正确的异常
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(HttpMediaTypeNotSupportedException.class)
    public Response<Object> handler(HttpMediaTypeNotSupportedException e) {
        log.debug(String.format("请求错误（%s）: %s", e.getClass().getName(), e.getMessage()));
        return Response.error(ResponseCode.PARAM_WRONG, "请求头\"Contene-Type\"字段有误");
    }

    /**
     * 处理请求方法错误的情况
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(HttpRequestMethodNotSupportedException.class)
    public Response<Object> handler(HttpRequestMethodNotSupportedException e) {
        log.debug(String.format("请求错误（%s）: %s", e.getClass().getName(), e.getMessage()));
        return Response.error(ResponseCode.INVALID_REQUEST);
    }

    /**
     * 处理上传文件时不是Multipart方式
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(MultipartException.class)
    public Response<Object> handler(MultipartException e) {
        log.debug(String.format("请求错误（%s）: %s", e.getClass().getName(), e.getMessage()));
        return Response.error(ResponseCode.INVALID_REQUEST);
    }

    /**
     * 权限不足
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(NotRoleException.class)
    public Response<Object> handler(NotRoleException e) {
        return Response.error(ResponseCode.PERMISSION_DENIED);
    }

    /**
     * 没登陆
     *
     * @param e 异常
     * @return 统一响应
     */
    @ExceptionHandler(NotLoginException.class)
    public Response<Object> handler(NotLoginException e) {
        return Response.error(ResponseCode.NEED_LOGIN);
    }
}