/*
 * Class created by lensfrex.
 */

/*
 * Class created by lensfrex.
 */

/*
 * Class created by lensfrex.
 */

/*
 * Class created by lensfrex.
 */

package net.lensfrex.dscape.web.service.user;

import cn.dev33.satoken.stp.SaTokenInfo;
import cn.dev33.satoken.stp.StpUtil;
import net.lensfrex.dscape.dao.entity.UserBasic;
import net.lensfrex.dscape.dao.service.UserBasicService;
import net.lensfrex.dscape.dto.request.UserLoginRequestBody;
import net.lensfrex.dscape.dto.response.general.ResponseCode;
import net.lensfrex.dscape.enums.user.UserRoleEnum;
import net.lensfrex.dscape.enums.user.UserStatusEnum;
import net.lensfrex.dscape.exception.GlobalException;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.dscape.web.service.user.admin.InviteCode;
import net.lensfrex.dscape.cache.UserBasicCache;
import net.lensfrex.dscape.dao.entity.UserRole;
import net.lensfrex.dscape.dao.service.UserRoleService;
import net.lensfrex.dscape.dto.request.user.RegisterRequestBody;
import net.lensfrex.dscape.dto.response.data.user.RegisterResponseBody;
import org.mindrot.jbcrypt.BCrypt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.UUID;

@Slf4j
@Service
public class UserService {
    private final UserBasicCache userBasicCache;

    private final UserBasicService userBasicService;

    private final UserRoleService userRoleService;

    private final InviteCode inviteCode;

    @Autowired
    public UserService(UserBasicCache userBasicCache,
                       UserBasicService userBasicService,
                       UserRoleService userRoleService,
                       InviteCode inviteCode) {

        this.userBasicCache = userBasicCache;
        this.userBasicService = userBasicService;
        this.userRoleService = userRoleService;
        this.inviteCode = inviteCode;
    }

    public SaTokenInfo checkLogin() {
        return StpUtil.getTokenInfo();
    }

    public SaTokenInfo login(UserLoginRequestBody requestBody) {
        String username = requestBody.getUserName();
        String password = requestBody.getPassword();

        // 找缓存
        UserBasic userBasic = userBasicCache.getUserByUserName(username);
        if (userBasic == null) {
            userBasic = this.getUserAndCache(username);
        }

        // 验证密码
        if (userBasic != null && BCrypt.checkpw(password, userBasic.getPassword())) {
            // 检查用户状态
            if (userBasic.getStatus() == UserStatusEnum.BANNED.getCode()) {
                throw new GlobalException(ResponseCode.USER_WAS_BANNED);
            } else if (userBasic.getStatus() == UserStatusEnum.NOT_IDENTIFIED.getCode()) {
                throw new GlobalException(ResponseCode.USER_WAS_NOT_IDENTIFIED);
            }

            StpUtil.login(userBasic.getUid());
        } else {
            throw new GlobalException(ResponseCode.USERNAME_OR_PASSWORD_WRONG);
        }

        return StpUtil.getTokenInfo();
    }

    private UserBasic getUserAndCache(String username) {
        UserBasic userBasic = userBasicService.getUser(UserBasic::getUserName, username);
        this.cacheUser(userBasic);

        return userBasic;
    }

    private UserBasic getUserAndCacheByUid(String uid) {
        UserBasic userBasic = userBasicService.getUser(UserBasic::getUid, uid);
        this.cacheUser(userBasic);

        return userBasic;
    }

    private boolean cacheUser(UserBasic userBasic) {
        try {
            if (userBasic != null) {
                userBasicCache.save(userBasic);
            }
        } catch (Exception e) {
            log.info("将用户数据存入redis缓存时发生错误");
            log.debug("追踪：", e);

            return false;
        }

        return true;
    }

    @Transactional
    public RegisterResponseBody register(RegisterRequestBody requestBody, String registerIp) {
        if (userNameExists(requestBody.getUserName())) {
            throw new GlobalException(ResponseCode.USER_NAME_EXISTS);
        }

        RegisterResponseBody registerResponseBody = new RegisterResponseBody();

        // 注册码验证
        String inviteCode = requestBody.getInviteCode();
        if (inviteCode == null) {
            // 验证是否注册为初始管理员
            if (userBasicService.count() != 0) {
                throw new GlobalException(ResponseCode.PERMISSION_DENIED);
            }

            String uid = addUserToDatabase(requestBody.getUserName(), requestBody.getPassword(), "null", UserRoleEnum.ADMIN, registerIp);
            registerResponseBody.setUid(uid);
        } else {
            String superior = this.inviteCode.useInviteCode(inviteCode);

            String uid = addUserToDatabase(requestBody.getUserName(), requestBody.getPassword(), superior, UserRoleEnum.NORMAL_USER, registerIp);
            registerResponseBody.setUid(uid);
        }

        return registerResponseBody;
    }

    /**
     * 添加一个用户到数据库
     *
     * @param userName     用户名
     * @param password     密码（未用bcrypt加密的）
     * @param superior     上级管理员，本身是管理员时可为空
     * @param userRoleEnum 用户角色
     * @return 用户uid
     */
    protected String addUserToDatabase(String userName, String password, String superior, UserRoleEnum userRoleEnum, String userIp) {
        UserBasic userBasic = new UserBasic();

        String uid = UUID.randomUUID().toString();
        userBasic.setUid(uid);
        userBasic.setPassword(BCrypt.hashpw(password, BCrypt.gensalt()));
        userBasic.setUserName(userName);
        userBasic.setStatus(UserStatusEnum.NORMAL.getCode());
        userBasic.setSuperior(superior);
        userBasic.setCreateIp(userIp);
        userBasicService.save(userBasic);

        UserRole userRole = new UserRole();
        userRole.setRole(userRoleEnum.getCode());
        userRole.setUid(uid);
        userRoleService.save(userRole);

        return uid;
    }

    protected boolean userNameExists(String username) {
        return userBasicService.userNameExists(username);
    }

    public void modifyPassword(String uid, String oldPassword, String newPassword) {
        String loginUser = StpUtil.getLoginIdAsString();

        UserBasic userBasic = this.getUserAndCacheByUid(uid);
        if (userBasic == null) {
            return;
        }

        if (!loginUser.equals(uid)) {
            if (!loginUser.equals(userBasic.getSuperior())) {
                throw new GlobalException(ResponseCode.PERMISSION_DENIED);
            }
        }

        if (BCrypt.checkpw(userBasic.getPassword(), oldPassword)) {
            throw new GlobalException(ResponseCode.PERMISSION_DENIED);
        }

        String newHashedPassword = BCrypt.hashpw(newPassword, BCrypt.gensalt());

        userBasicService.modify(UserBasic::getUid, uid,
                UserBasic::getPassword, newHashedPassword);

        userBasicCache.clear(uid);
    }
}